##################################
###PARABOLIC CHAPOTON TRIANGLES###
##################################

def alpha_m_triangle(alpha,alternate=true):
	r"""
	Constructs the generating function of the M"obius function of either the noncrossing alpha-partition poset, or of the core label order of the alpha-Tamari lattice.
	
	The *parabolic M-triangle* is the generating function of the M"obius function of the core label order of the alpha-Tamari lattice, see Section~6 in [Mue18]_.  This core label order is in general a subposet of the poset of noncrossing alpha-partitions.
	
	:param alpha: an integer composition.
	:param alternate: a boolean variable.
	
	EXAMPLES::

		sage: alpha_m_triangle((1,2,1))
		4*x^2*y^2 - 9*x^2*y + 5*x^2 + 5*x*y - 5*x + 1
		sage: alpha_m_triangle((1,2,1), alternate=false)
		4*x^2*y^2 - 10*x^2*y + 6*x^2 + 5*x*y - 5*x + 1

	REFERENCES::
		.. [Mue18] Henri M\"uhle.
		   *Noncrossing Partitions, Tamari Lattices, and Parabolic Quotients of the Symmetric Group*
		   arXiv:1809.01405, 2018.
	"""
	Z.<x,y> = PolynomialRing(ZZ)
	f = 0
	if alternate:
		P = alpha_core_label_order(alpha)
	else:
		P = alpha_partition_poset(alpha,true)
	for p1 in P:
		i = P.rank(p1)
		for p2 in P:
			j = P.rank(p2)
			f += P.moebius_function(p1,p2)*x^i*y^j
	return f
	
def alpha_h_triangle(alpha):
	r"""
	Constructs the bivariate polynomial that counts antichains in the parabolic root poset according to total size and the number of simple transpositions.
	
	The *parabolic H-triangle* is the polynomial in two variables, which is defined as the sum over all antichains in the parabolic root poset, where the first variable records the size of the antichain, and the second variable records the number of simple transpositions contained in this antichain.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: parabolic_h_triangle((1,2,1))
		x^2*y^2 + 2*x^2*y + x^2 + 2*x*y + 3*x + 1
	"""
	Z.<x,y> = PolynomialRing(ZZ)
	f = 0
	R = alpha_root_poset(alpha)
	T = R.antichains()
	M = R.minimal_elements()
	for A in T:
		i = len(A)
		j = Set(A).intersection(Set(M)).cardinality()
		f += x^i*y^j
	return f
	
def alpha_f_triangle(alpha,orig='h'):
	r"""
	Constructs a rational function that can be obtained by plugging certain values into the parabolic H-triangle.
	
	The *parabolic F-triangle* is the rational function which is a particular evaluation of the parabolic H-triangle, respectively the parabolic M-triangle, see Section~6 of [Mue18]_.
	
	:param alpha: an integer composition.
	:param orig: a string.
	
	EXAMPLES::
	
		sage: alpha_f_triangle((1,2,1))
		5*x^2 + 4*x*y + y^2 + 9*x + 4*y + 4
		sage: alpha_f_triangle((2,1,2))
		(14*x^3 + 4*x^2*y + x*y^2 + 25*x^2 + 4*x*y + 12*x + 1)/x
	
	REFERENCES::
		.. [Mue18] Henri M\"uhle.
		   *Noncrossing Partitions, Tamari Lattices, and Parabolic Quotients of the Symmetric Group*
		   arXiv:1809.01405, 2018.
	"""
	Z.<x,y> = PolynomialRing(ZZ)
	T = alpha_tamari_lattice(alpha)
	r = len(alpha)-1
	if orig=='h':
		ph = alpha_h_triangle(alpha)
		return x^r*ph((x+1)/x,(y+1)/(x+1))
	if orig=='m':
		pm = alpha_m_triangle(alpha,true)
		return y^r*pm((y+1)/(y-x),(y-x)/y)

def alpha_h_m_conjecture(alpha,alternate=true):
	r"""
	This checks whether the parabolic M- and H-triangle are related by certain substitutions for the given composition `alpha`
	
	Conjecture~1.9 in [Mue18]_ states that we can obtain the parabolic M-triangle by certain substitutions from the parabolic H-triangle if and only if `alpha` has a unique part which is bigger than `1`.  This function checks this conjecture for the given composition `alpha`.  It may also use the fake M-triangle defined on the poset of noncrossing alpha-partitions.
	
	:param alpha: an integer composition.
	:param alternate: a Boolean variable.
	
	EXAMPLES::
	
		sage: alpha_h_m_conjecture((1,2,1))
		True
		sage: alpha_h_m_conjecture((1,2,1), alternate=false)
		False
		sage: alpha_h_m_conjecture((2,1,2))
		False
		sage: alpha_h_m_conjecture((2,1,2), alternate=false)
		False
	
	REFERENCES::
		.. [Mue18] Henri M\"uhle.
		   *Noncrossing Partitions, Tamari Lattices, and Parabolic Quotients of the Symmetric Group*
		   arXiv:1809.01405, 2018.
	"""
	Z.<x,y> = PolynomialRing(ZZ)
	pm = alpha_m_triangle(alpha,alternate)
	ph = alpha_h_triangle(alpha)
	r = len(alpha)-1
	
	h = ph(x,y)
	m = (1+x*(y-1))^r*pm(y/(y-1),(x*(y-1))/(x*(y-1)+1))
	return (h==m)
