##########################
###PARABOLIC DYCK PATHS###
##########################

def dyck_to_height(w):
	r"""
	Constructs the height sequence of the Dyck path 'w'.
	
	The *height sequence* of a Dyck path `w` of semilength `n` is the n-tuple  `(h_1,h_2,\ldots,h_n)`, where `h_i` is the y-coordinate of `w` at `x=i-1`.
	
	:param w: a Dyck path.
	
	EXAMPLES::
	
		sage: w = DyckWords(4).an_element()
		sage: w.pretty_print()
		       _
		     _|
		   _|  .
		 _|  . .
		|  . . .
		
		sage: dyck_to_height(w)
		[1, 2, 3, 4]
	"""
	return ([sum(w[:i]) for i in range(1,len(w)) if w[i]==0])
	
def bounce_path(alpha):
	r"""
	Constructs the bounce path corresponding to the integer partition 'alpha'.
	
	The *bounce path* of an integer compositon `(a_1,a_2,\ldots,a_r)` as defined in Section 1.1.3 of [CFM19]_ is the Dyck path `N^{a_1}E^{a_1}N^{a_2}E^{a_2}\cdots N^{a_r}E^{a_r}`.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: w = bounce_path((3,2,5,1))
		sage: w.pretty_print()
		                     _
		           _________|
		          | x x x x  .
		          | x x x  . .
		          | x x  . . .
		          | x  . . . .
		       ___|  . . . . .
		      | x  . . . . . .
		 _____|  . . . . . . .
		| x x  . . . . . . . .
		| x  . . . . . . . . .
		|  . . . . . . . . . .

	REFERENCES::
	
		.. [CFM19] Cesar Ceballos, Wenjie Fang, Henri M\"uhle.
		   *The Steep-Bounce Zeta Map in Parabolic Cataland*
		   arXiv:1903.08515, 2019.
	"""
		
	w = []
	for i in range(len(alpha)):
		w += alpha[i]*[1] + alpha[i]*[0]
	return DyckWord(w)
	
def alpha_dyck_paths(alpha):
	r"""
	Returns the set of all alpha-Dyck paths with respect to the composition 'alpha'.
		
	An *alpha-Dyck path* with respect to an integer composition `\alpha` as defined in Section~1.1.3 of [CFM19]_ is a Dyck path of length `sum(\alpha)` which stays weakly above the bounce path of `\alpha`.
		
	:param alpha: an integer composition.
				
	EXAMPLES::
	
	sage: alpha_dyck_paths((1,1,1))
	[[1, 0, 1, 0, 1, 0],
	 [1, 0, 1, 1, 0, 0],
	 [1, 1, 0, 0, 1, 0],
	 [1, 1, 0, 1, 0, 0],
	 [1, 1, 1, 0, 0, 0]]
	sage: alpha_dyck_paths((2,1))
	 [[1, 1, 0, 0, 1, 0], [1, 1, 0, 1, 0, 0], [1, 1, 1, 0, 0, 0]]

	REFERENCES::
	
		.. [CFM19] Cesar Ceballos, Wenjie Fang, Henri M\"uhle.
		   *The Steep-Bounce Zeta Map in Parabolic Cataland*
		   arXiv:1903.08515, 2019.
	"""

	n = sum(alpha)
	bnc = bounce_path(alpha)
	return [w for w in DyckWords(n) if componentwise_leq(dyck_to_height(bnc),dyck_to_height(w))]
	
def area_dinv_enumeration(alpha):
	r"""
	Returns the bivariate generating function of the area- and dinv-statistics with respect to all alpha-Dyck paths.  The variable `s` counts area, and `t` counts dinv.
		
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: area_dinv_enumeration((1,2,1))
		x^6 + x^5*y + x^4*y^2 + x^3*y^3 + x^2*y^4 + x^4*y + x^3*y^2 + x*y^4 + x^3*y + x^2*y^2
	"""
		
	Q.<x,y> = PolynomialRing(QQ)
	pl = Q.zero()
	for w in alpha_dyck_paths(alpha):
		pl += x^w.area()*y^w.dinv()
	return pl

def alpha_root_poset(alpha):
	r"""
	Constructs the parabolic root poset as defined in Section~5 of [MW18]_.
	
	The parabolic root poset with respect to `alpha` is the order filter in the triangular poset of parameter `n=sum(alpha)` generated by all simple transpositions `(i,i+1)` for `i=alpha[0]+alpha[1]+\dots+alpha[k]` for some `k`.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_root_poset((1,2,1))
		Finite poset containing 5 elements
	
	REFERENCES::
	
		.. [MW18] Henri M\"uhle, Nathan Williams.  
		   *Tamari Lattices for Parabolic Quotients of the Symmetric Group*
		   arXiv:1804.02761, 2018.
	"""	
	T = triangular_poset(sum(alpha))
	return T.subposet(T.order_filter([(sum(alpha[:i]),sum(alpha[:i])+1) for i in range(1,len(alpha))]))

def alpha_nonnesting_partitions(alpha):
	r"""
	Constructs nonnesting alpha-partitions.
	
	A nonnesting alpha-partition is an antichain in the parabolic root poset.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_nonnesting_partitions((1,2,1)).list()
		[[],
		 [(3, 4)],
		 [(3, 4), (1, 2)],
		 [(3, 4), (1, 3)],
		 [(2, 4)],
		 [(2, 4), (1, 2)],
		 [(2, 4), (1, 3)],
		 [(1, 2)],
		 [(1, 3)],
		 [(1, 4)]]
	"""
	return alpha_root_poset(alpha).antichains()
	
def alpha_ferrers(alpha):
	r"""
	Returns the shape of the parabolic root poset as an integer partition.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_ferrers((1,2,1))
		[3, 1, 1]
		sage: alpha_ferrers((2,1,2))
		[3, 3, 2]
	"""
	l = []
	for i in range(1,len(alpha)):
		l += alpha[i]*[sum(alpha[:i])]
	m = len(l)
	return [l[m-1-i] for i in range(len(l))]

def alpha_size_matrix(alpha):
	r"""
	Returns the matrix whose determinant equals the cardinality of the set of alpha-Dyck paths.
	
	The number of Ferrers shapes fitting inside a given Ferrers shape can be computed via the determinant of a matrix consisting of suitable binomial coefficients, see Section~2.3.7 of [Kr65].
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_size_matrix((1,2,1))
		[4 1 0]
		[1 2 1]
		[0 1 2]
		sage: alpha_size_matrix((1,1,1,1))
		[4 3 0]
		[1 3 1]
		[0 1 2]
	
	REFERENCES::
	
		.. [Kr65] Germain Kreweras.
		   *Sur une classe de probl`emes de d'enombrement li'es au treillis des entiers*
		   Cahiers du B.U.R.O. Univ. de Rech. Op'er *6*, 1965.
	"""
	frs = alpha_ferrers(alpha)
	m = len(frs)
	M = matrix(m)
	for i in range(m):
		row = []
		for j in range(m):
			row.append(binomial(frs[j]+1,j-i+1))
		M.set_row(i,row)
	return M

def alpha_cardinality(alpha):
	r"""
	Returns the number of alpha-Dyck paths.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: alpha_cardinality((1,2,1))
		10
		sage: alpha_cardinality((1,1,1,1))
		14
		
	"""
	return alpha_size_matrix(alpha).determinant()
