#########################
###AUXILIARY FUNCTIONS###
#########################

def componentwise_leq(u,v):
	r"""
	Checks if the given sequences are componentwise comparable.
		
	:params u: an integer tuple.
	:params v: an integer tuple.
	
	EXAMPLES::
	
		sage: componentwise_leq((1,2,3),(3,3,3))
		True
		sage: componentwise_leq((1,2,3),(3,1,3))
		False
		
	"""
	assert(len(u)==len(v))
	return all([u[i]<=v[i] for i in range(len(u))])
	
def is_increasing(list):
	"""
	Checks if a given list of integers is ordered in increasing order.
	
	:params list: a list of integers.
	
	EXAMPLES::
	
		sage: is_increasing([1,2,3,4])
		True
		sage: is_increasing([1,2,4,3])
		False
	"""
	return all([list[i]<list[i+1] for i in range(len(list)-1)])

def create_alpha_regions(alpha):
	"""
	Creates a partition of a set.
	
	Let `n=sum(alpha)`.  This function splits the set `\{1,2,\ldots,n\}` according to the parts of the composition alpha.
	
	:param alpha: an integer composition.
	
	EXAMPLES::
	
		sage: create_alpha_regions((1,2,4,1))
		[[1], [2, 3], [4, 5, 6, 7], [8]]
	"""
	return [range(sum(alpha[:i])+1,sum(alpha[:i+1])+1) for i in range(len(alpha))]
	
def region_index(alpha,i):
	"""
	Returns to which alpha-region the integer i belongs.
	
	:param alpha: an integer composition.
	
	:param i: an integer.
	
	EXAMPLES::
	
		sage: region_index((1,2,1),2)
		2
	"""
	aregs = create_alpha_regions(alpha)
	ind = -1
	for k in range(len(aregs)):
		if i in aregs[k]:
			ind = k+1
			break
	return ind
	
def split_permutation(p,alpha):
	"""
	Splits a permutation according to an integer composition
	
	This function splits the one-line notation of a permutation into parts according to a given integer composition
	
	:params p: a permutation.
	:params alpha: an integer composition
	
	EXAMPLES::
	
		sage: alpha = (1,4,3,2)
		sage: S = SymmetricGroup(sum(alpha))
		sage: p = Permutation(S.an_element()); p
		[1, 3, 4, 5, 6, 7, 8, 9, 10, 2]
		sage: split_permutation(p,alpha)
		[[1], [3, 4, 5, 6], [7, 8, 9], [10, 2]]
	"""
	return [[p[j] for j in range(sum(alpha[:i]),sum(alpha[:i+1]))] for i in range(len(alpha))]
	
def triangular_poset(n):
	r"""
	Constructs a triangular poset.
	
	The triangular poset of parameter `n` consists of all pairs `(i,j)` with `1 \leq i < j \leq n` ordered by `(a,b) \leq (c,d)` if and only if `a \geq c` and `b \leq d`.
	
	:param n: an integer.
	
	EXAMPLES::
	
		sage: triangular_poset(4)
		Finite poset containing 6 elements
	"""
	
	pairs = [x for x in cartesian_product([range(1,n+1),range(1,n+1)]) if x[0]<x[1]]
	transposition_order = lambda u,v:u[0]>=v[0] and u[1]<=v[1]
	return Poset([pairs,transposition_order])
